/*
 * Copyright (c) 2011, Andreas Spall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * - Neither the name of Andreas Spall nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software 
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.osoboo.jira.metadata.ui.field;

import java.util.Map;

import org.apache.velocity.tools.generic.MathTool;

import com.atlassian.jira.ComponentManager;
import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.customfields.manager.GenericConfigManager;
import com.atlassian.jira.issue.customfields.persistence.CustomFieldValuePersister;
import com.atlassian.jira.issue.fields.CustomField;
import com.atlassian.jira.security.JiraAuthenticationContext;
import com.atlassian.jira.util.JiraVelocityUtils;
import com.atlassian.plugin.web.WebFragmentHelper;
import com.osoboo.jira.metadata.MetadataService;

@SuppressWarnings("unchecked")
public class MetadataCalcFieldType extends MetadataCalculatedCFType {

	private final WebFragmentHelper webFragmentHelper;
	private MetadataService metadataService;

	protected MetadataCalcFieldType(
			CustomFieldValuePersister customFieldValuePersister,
			GenericConfigManager genericConfigManager,
			JiraAuthenticationContext authenticationContext,
			WebFragmentHelper webFragmentHelper, MetadataService metadataService) {
		super(customFieldValuePersister, genericConfigManager);
		this.webFragmentHelper = webFragmentHelper;
		this.metadataService = metadataService;
	}


	@Override
	public Object getValueFromIssue(CustomField arg0, Issue arg1) {
		try {
			final Map<String, Object> startingParams = JiraVelocityUtils
					.getDefaultVelocityParams(ComponentManager.getInstance()
							.getJiraAuthenticationContext());
			startingParams.put("issue", arg1);
			startingParams.put("customField", arg0);
			startingParams.put("metadataService", metadataService);
			startingParams.put("math", new MathTool());
			return webFragmentHelper.renderVelocityFragment(
					getDefaultValue(arg0.getRelevantConfig(arg1)).toString(),
					startingParams);
		} catch (Exception e) {
			return "";
		}
	}





}
