/*
 * Copyright (c) 2011, Andreas Spall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * - Neither the name of Andreas Spall nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software 
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.osoboo.jira.metadata.ui.field;

import java.util.List;

import com.atlassian.core.util.collection.EasyList;
import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.customfields.config.item.DefaultValueConfigItem;
import com.atlassian.jira.issue.customfields.impl.CalculatedCFType;
import com.atlassian.jira.issue.customfields.impl.FieldValidationException;
import com.atlassian.jira.issue.customfields.manager.GenericConfigManager;
import com.atlassian.jira.issue.customfields.persistence.CustomFieldValuePersister;
import com.atlassian.jira.issue.customfields.persistence.PersistenceFieldType;
import com.atlassian.jira.issue.fields.CustomField;
import com.atlassian.jira.issue.fields.config.FieldConfig;
import com.atlassian.jira.issue.fields.config.FieldConfigItemType;
import com.atlassian.jira.util.JiraUtils;
import com.atlassian.jira.util.collect.CollectionBuilder;

@SuppressWarnings("unchecked")
public abstract class MetadataCalculatedCFType extends CalculatedCFType {

	private CustomFieldValuePersister customFieldValuePersister;
	private GenericConfigManager genericConfigManager;

	protected MetadataCalculatedCFType(
			CustomFieldValuePersister customFieldValuePersister,
			GenericConfigManager genericConfigManager) {
		this.customFieldValuePersister = customFieldValuePersister;
		this.genericConfigManager = genericConfigManager;
	}

	@Override
	public Object getSingularObjectFromString(String arg0)
			throws FieldValidationException {
		return arg0;
	}

	@Override
	public String getStringFromSingularObject(Object arg0) {
		// TODO Auto-generated method stub
		if (arg0 == null) {
			return "";
		}
		return arg0.toString();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List getConfigurationItemTypes() {
		return CollectionBuilder.<FieldConfigItemType> newBuilder(
				JiraUtils.loadComponent(DefaultValueConfigItem.class))
				.asArrayList();
	}

	public void createValue(CustomField field, Issue issue, Object value) {
		customFieldValuePersister.createValues(field, issue.getId(),
				getDatabaseType(), EasyList.build(getDbValueFromObject(value)));
	}

	protected PersistenceFieldType getDatabaseType() {
		return PersistenceFieldType.TYPE_LIMITED_TEXT;
	}

	/**
	 * the value does exist, and the new value is different than the existing
	 * one.
	 */
	public void updateValue(CustomField customField, Issue issue, Object value) {
		customFieldValuePersister.updateValues(customField, issue.getId(),
				getDatabaseType(), EasyList.build(getDbValueFromObject(value)));
	}

	private Object getDbValueFromObject(Object value) {
		return value;
	}

	public void setDefaultValue(FieldConfig fieldConfig, Object value) {
		genericConfigManager.update(DEFAULT_VALUE_TYPE, fieldConfig.getId()
				.toString(), getDbValueFromObject(value));
	}

	public Object getDefaultValue(FieldConfig fieldConfig) {
		try {
			Object databaseValue = genericConfigManager.retrieve(
					DEFAULT_VALUE_TYPE, fieldConfig.getId().toString());
			if (databaseValue != null) {
				return getObjectFromDbValue(databaseValue);
			} else {
				return null;
			}
		} catch (FieldValidationException e) {
			return null;
		}
	}

	private Object getObjectFromDbValue(Object databaseValue) {
		return databaseValue;
	}

}
